@@ -60,7 +60,7 @@ build_default_formats = {
         "row": 2  # This is how many rows we shift from the top,
     },
 
-    "set_page_setup": {}
+    "set_page_setup": {"hide_gridlines":{"option":2}}
 
 #    "set_rows":[
 #        {"row":6,
